/*
 * ffmpeg_stub.c
 * Copyright (C) 2010 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <windows.h>
#include "ffmpeg_stub.h"

int ffmpeg_stub_valid=0;

struct ffmpeg_vs ffmpeg_vs;
struct av_stub av;
struct avcodec_stub avcodec;
struct avpicture_stub avpicture;
struct sws_stub sws;

static HINSTANCE hLibAvUtil=NULL;
static HINSTANCE hLibAvCodec=NULL;
static HINSTANCE hLibAvFormat=NULL;
static HINSTANCE hLibSwScale=NULL;

int mp3_decoder_valid()
  { return NULL!=av.mp3_decoder; }
int ac3_decoder_valid()
  { return NULL!=av.ac3_decoder; }
int flac_decoder_valid()
  { return NULL!=av.flac_decoder; }

#define DECL(lib, pfx1, pfx2, sep, id) \
  if (NULL==(pfx1.id=(void *)GetProcAddress(hLib##lib, #pfx2 #sep #id))) \
    goto fail

int ffmpeg_stub_load(const char *path, char *p)
{
  strcpy(p, "in_ffsox\\avutil-50.dll");

  if (NULL==(hLibAvUtil=LoadLibrary(TEXT(path))))
    goto fail;

  DECL(AvUtil,ffmpeg_vs,,,avutil_version);
  DECL(AvUtil,av,av,_,free);
  DECL(AvUtil,av,av,_,malloc);
  DECL(AvUtil,av,av,_,mallocz);

  strcpy(p, "in_ffsox\\avcodec-52.dll");

  if (NULL==(hLibAvCodec=LoadLibrary(TEXT(path))))
    goto fail;

  DECL(AvCodec,ffmpeg_vs,,,avcodec_version);

/*
  DECL(AvCodec,av,,,ac3_decoder);
  DECL(AvCodec,av,,,mp3_decoder);
  DECL(AvCodec,av,,,flac_decoder);
*/
  av.ac3_decoder=(void *)GetProcAddress(hLibAvCodec, "ac3_decoder");
  av.mp3_decoder=(void *)GetProcAddress(hLibAvCodec, "mp3_decoder");
  av.flac_decoder=(void *)GetProcAddress(hLibAvCodec, "flac_decoder");
  DECL(AvCodec,av,av,_,dup_packet);
  DECL(AvCodec,av,av,_,free_packet);
  DECL(AvCodec,av,av,_,picture_copy);

  DECL(AvCodec,avcodec,avcodec,_,alloc_frame);
  DECL(AvCodec,avcodec,avcodec,_,close);
  DECL(AvCodec,avcodec,avcodec,_,decode_audio3);
  DECL(AvCodec,avcodec,avcodec,_,decode_video2);
  DECL(AvCodec,avcodec,avcodec,_,find_decoder);
  DECL(AvCodec,avcodec,avcodec,_,open);

  DECL(AvCodec,avpicture,avpicture,_,alloc);
  DECL(AvCodec,avpicture,avpicture,_,deinterlace);
  DECL(AvCodec,avpicture,avpicture,_,free);

  strcpy(p, "in_ffsox\\avformat-52.dll");

  if (NULL==(hLibAvFormat=LoadLibrary(TEXT(path))))
    goto fail;

  DECL(AvFormat,ffmpeg_vs,,,avformat_version);

  DECL(AvFormat,av,av,_,iformat_next);
  DECL(AvFormat,av,av,_,close_input_file);
  DECL(AvFormat,av,av,_,find_stream_info);
  DECL(AvFormat,av,av,_,metadata_get);
  DECL(AvFormat,av,av,_,open_input_file);
  DECL(AvFormat,av,av,_,read_frame);
  DECL(AvFormat,av,av,_,register_all);
  DECL(AvFormat,av,,,dump_format);
  DECL(AvFormat,av,,,url_set_interrupt_cb);

  strcpy(p, "in_ffsox\\swscale-0.dll");

  if (NULL==(hLibSwScale=LoadLibrary(TEXT(path))))
    goto fail;

  DECL(SwScale,ffmpeg_vs,,,swscale_version);

  DECL(SwScale,sws,sws,_,freeContext);
  DECL(SwScale,sws,sws,_,getContext);
  DECL(SwScale,sws,sws,_,scale);

  return ffmpeg_stub_valid=1;
fail:
  return ffmpeg_stub_unload();
}

#undef DECL

int ffmpeg_stub_unload()
{
  if (NULL!=hLibSwScale) {
    FreeLibrary(hLibSwScale);
    hLibSwScale=NULL;
  }

  if (NULL!=hLibAvFormat) {
    FreeLibrary(hLibAvFormat);
    hLibAvFormat=NULL;
  }

  if (NULL!=hLibAvCodec) {
    FreeLibrary(hLibAvCodec);
    hLibAvCodec=NULL;
  }

  if (NULL!=hLibAvUtil) {
    FreeLibrary(hLibAvUtil);
    hLibAvUtil=NULL;
  }

  return ffmpeg_stub_valid=0;
}
